﻿//========================================================================================
// "Выборка текста по стилю абзаца"
//       (FindTextStyle.js)
//
// Версия 1.0
//
//    Скрипт пробегает по заданному диапазону страниц, переберает все текстовые фреймы
// и выбирает из них текст указанного стиля абзаца. Рассчитывается площадь фрейма.
//    Результат работы сохраняется в текстовый файл.
//
// 1.1: исправлен косяк с номерами страниц.
//    
// Автор: Андрей Рыжков (ragman)
//
// Дата: 26.03.2005 г.
//
//=========================================================================================
scriptName = "Выборка текста по стилю абзаца";
scriptVer = " 1.1";
txtFile = "";

//Функция записи файла
function writeFile (myFile, myString) {
	var myResult = myFile.open("w");
	if (!myResult) {
		alert("Невозможно открыть файл для записи!");
		exit();
	}
	myFile.write(myString);
	myFile.close();
}

//Функция проверки имени файла
function fileNameCheck (myFile, myExt) {
	var myPath = myFile.path;
	var myName = myFile.name;
	if (myName.substring((myName.length - myExt.length), ) != myExt) {
		myName +=myExt;
	}
	return new File (myPath + "/" + myName);
}

with (app) {
	
	// определяем количество страниц в документе
	myDoc = activeDocument;
	myPageCount = myDoc.pages.length;
	
	// Устанавливаем единицы измерения в мм
	myDoc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.millimeters;
	myDoc.viewPreferences.verticalMeasurementUnits = MeasurementUnits.millimeters;
	
	//Получаем список стилей	
	var myStyleNames = new Array();
	for (myCounter = 0; myCounter < myDoc.paragraphStyles.length; myCounter++) {
		myStyleNames[myCounter] = myDoc.paragraphStyles[myCounter].name;
	}

	
// Морда лица
	// Создаем диалог
	var myDialog = dialogs.add({name:scriptName + scriptVer});
	// Подготовка
	with (myDialog.dialogColumns.add()) {
		with (dialogRows.add().borderPanels.add()) {
			with (dialogColumns.add()) {
				dialogRows.add().staticTexts.add({staticLabel:"Начальная страница:"});
				dialogRows.add().staticTexts.add({staticLabel:"Конечная страница:"});
				dialogRows.add().staticTexts.add({staticLabel:"Искомый стиль:"});
			}
			with (dialogColumns.add()) {
				var myDialogPageStart = dialogRows.add().integerEditboxes.add({editValue:1});
				var myDialogPageEnd = dialogRows.add().integerEditboxes.add({editValue:myPageCount});
				var mySelectStyle = dialogRows.add().dropdowns.add({stringList:myStyleNames, selectedIndex:0});
			}
		}
		with (dialogRows.add()) {
			dialogColumns.add().staticTexts.add({staticLabel:"Андрей Рыжков     special946@gmail.com"});
		}
	}
	// Показываем
	var myResult = myDialog.show();
	if (!myResult) {
		myDialog.destroy();
		exit();
	}
	// проверяем правильность ввода страниц
	myPageStart = myDialogPageStart.editValue;
	myPageEnd = myDialogPageEnd.editValue;
	if (myPageEnd < myPageStart) {
		alert ("Шутишь?");
		exit();

	}
	findParaStyle = myDoc.paragraphStyles[mySelectStyle.selectedIndex].name;
	
	// Проходимся по страницам
	for (myCount_1 = myPageStart - 1; myCount_1 < myPageEnd; myCount_1++) {
		myPage = myDoc.pages.item(myCount_1);
		// Проходимся по текстовым фреймам на странице
		for (myCount_2 = 0; myCount_2 < myPage.textFrames.length; myCount_2++) {
			myTextFrame = myPage.textFrames.item(myCount_2);
			// geometricBounds [y1, x1, y2, x2]
			myFrameLenth = myTextFrame.geometricBounds[2] - myTextFrame.geometricBounds[0];
			myFrameHeight = myTextFrame.geometricBounds[3] - myTextFrame.geometricBounds[1];
			plaseFrame = myFrameLenth * myFrameHeight
			// Пробегаем по абзацам во фрейме
			for (myCount_3 = 0; myCount_3 < myTextFrame.paragraphs.length; myCount_3++) {
				myParagraph = myTextFrame.paragraphs.item(myCount_3);
				// Сравниваем стиль текущего абзаца с заданным
				if (myParagraph.appliedParagraphStyle.name == findParaStyle) {
					txtFile = txtFile + "\"" + myParagraph.contents + "\"\n" + "Страница " + (myCount_1 + 1) + "  Площадь=" + plaseFrame + "кв.мм\n" + "======================\n"
// для экселя				txtFile = txtFile + myParagraph.contents + "\t" + (myCount_1 + 1) + "\t" + plaseFrame + "\n"
				}
			}
		}
	}
	
	// Пишем файл
	var myFile = File.saveDialog(scriptName + " " + scriptVer + ". Сохранение результата", "TXT:*.txt");
	if (!myFile) {
		alert("Файл не выбран!");
		exit();
	}
	myFile = fileNameCheck(myFile, ".txt");
	writeFile(myFile, txtFile);
	alert("Данные успешно сохранены");
}